@@ -57,24 +57,27 @@ impl Action for AlterColumn {
 
         // Add temporary, nullable column
         let mut temp_column_definition_parts: Vec<&str> =
-            vec![&temporary_column_name, &temporary_column_type];
+            vec![&temporary_column_name, temporary_column_type];
 
         // Use either new default value or existing one if one exists
-        let default_value = self.changes.default.as_ref().or(column.default.as_ref());
+        let default_value = self
+            .changes
+            .default
+            .as_ref()
+            .or_else(|| column.default.as_ref());
         if let Some(default) = default_value {
             temp_column_definition_parts.push("DEFAULT");
             temp_column_definition_parts.push(default);
         }
 
         let query = format!(
-            "
-			ALTER TABLE {table}
+            r#"
+			ALTER TABLE "{table}"
             ADD COLUMN IF NOT EXISTS {temp_column_definition}
-			",
+			"#,
             table = self.table,
             temp_column_definition = temp_column_definition_parts.join(" "),
         );
-        println!("Query {}", query);
         db.run(&query).context("failed to add temporary column")?;
 
         // If up or down wasn't provided, we default to simply moving the value over.
@@ -97,7 +100,7 @@ impl Action for AlterColumn {
             .collect();
 
         let query = format!(
-                "
+            r#"
                 CREATE OR REPLACE FUNCTION {up_trigger}()
                 RETURNS TRIGGER AS $$
                 BEGIN
@@ -113,8 +116,8 @@ impl Action for AlterColumn {
                 END
                 $$ language 'plpgsql';
 
-                DROP TRIGGER IF EXISTS {up_trigger} ON {table};
-                CREATE TRIGGER {up_trigger} BEFORE INSERT OR UPDATE ON {table} FOR EACH ROW EXECUTE PROCEDURE {up_trigger}();
+                DROP TRIGGER IF EXISTS "{up_trigger}" ON "{table}";
+                CREATE TRIGGER "{up_trigger}" BEFORE INSERT OR UPDATE ON "{table}" FOR EACH ROW EXECUTE PROCEDURE {up_trigger}();
 
                 CREATE OR REPLACE FUNCTION {down_trigger}()
                 RETURNS TRIGGER AS $$
@@ -131,19 +134,19 @@ impl Action for AlterColumn {
                 END
                 $$ language 'plpgsql';
 
-                DROP TRIGGER IF EXISTS {down_trigger} ON {table};
-                CREATE TRIGGER {down_trigger} BEFORE INSERT OR UPDATE ON {table} FOR EACH ROW EXECUTE PROCEDURE {down_trigger}();
-                ",
-                existing_column = &self.column,
-                existing_column_real = column.real_name,
-                temp_column = self.temporary_column_name(ctx),
-                up = up,
-                down = down,
-                table = self.table,
-                up_trigger = self.up_trigger_name(ctx),
-                down_trigger = self.down_trigger_name(ctx),
-                declarations = declarations.join("\n"),
-            );
+                DROP TRIGGER IF EXISTS "{down_trigger}" ON "{table}";
+                CREATE TRIGGER "{down_trigger}" BEFORE INSERT OR UPDATE ON "{table}" FOR EACH ROW EXECUTE PROCEDURE {down_trigger}();
+                "#,
+            existing_column = &self.column,
+            existing_column_real = column.real_name,
+            temp_column = self.temporary_column_name(ctx),
+            up = up,
+            down = down,
+            table = self.table,
+            up_trigger = self.up_trigger_name(ctx),
+            down_trigger = self.down_trigger_name(ctx),
+            declarations = declarations.join("\n"),
+        );
         db.run(&query)
             .context("failed to create up and down triggers")?;
 
@@ -151,17 +154,44 @@ impl Action for AlterColumn {
         common::batch_touch_rows(db, &table.real_name, &column.real_name)
             .context("failed to batch update existing rows")?;
 
+        // Duplicate any indices to the temporary column
+        let indices = common::get_indices_for_column(db, &table.real_name, &column.real_name)?;
+        for (index_name, index_oid) in indices {
+            let index_columns: Vec<String> = common::get_index_columns(db, &index_name)?
+                .into_iter()
+                .map(|idx_column| {
+                    // Replace column with temporary column for new index
+                    if idx_column == column.real_name {
+                        temporary_column_name.to_string()
+                    } else {
+                        idx_column
+                    }
+                })
+                .collect();
+            let temp_index_name = self.temp_index_name(ctx, index_oid);
+
+            db.query(&format!(
+                r#"
+                CREATE INDEX CONCURRENTLY IF NOT EXISTS "{new_index_name}" ON "{table}" ({columns})
+                "#,
+                new_index_name = temp_index_name,
+                table = table.real_name,
+                columns = index_columns.join(", "),
+            ))
+            .context("failed to create temporary index")?;
+        }
+
         // Add a temporary NOT NULL constraint if the column shouldn't be nullable.
         // This constraint is set as NOT VALID so it doesn't apply to existing rows and
         // the existing rows don't need to be scanned under an exclusive lock.
         // Thanks to this, we can set the full column as NOT NULL later with minimal locking.
         if !column.nullable {
             let query = format!(
-                "
-                ALTER TABLE {table}
-                ADD CONSTRAINT {constraint_name}
-                CHECK ({column} IS NOT NULL) NOT VALID
-                ",
+                r#"
+                ALTER TABLE "{table}"
+                ADD CONSTRAINT "{constraint_name}"
+                CHECK ("{column}" IS NOT NULL) NOT VALID
+                "#,
                 table = self.table,
                 constraint_name = self.not_null_constraint_name(ctx),
                 column = self.temporary_column_name(ctx),
@@ -181,10 +211,10 @@ impl Action for AlterColumn {
         if self.can_short_circuit() {
             if let Some(new_name) = &self.changes.name {
                 let query = format!(
-                    "
-			        ALTER TABLE {table}
-			        RENAME COLUMN {existing_name} TO {new_name}
-			        ",
+                    r#"
+			        ALTER TABLE "{table}"
+			        RENAME COLUMN "{existing_name}" TO "{new_name}"
+			        "#,
                     table = self.table,
                     existing_name = self.column,
                     new_name = new_name,
@@ -210,10 +240,10 @@ impl Action for AlterColumn {
             // Validate the temporary constraint (should always be valid).
             // This performs a sequential scan but does not take an exclusive lock.
             let query = format!(
-                "
-                ALTER TABLE {table}
-                VALIDATE CONSTRAINT {constraint_name}
-                ",
+                r#"
+                ALTER TABLE "{table}"
+                VALIDATE CONSTRAINT "{constraint_name}"
+                "#,
                 table = self.table,
                 constraint_name = self.not_null_constraint_name(ctx),
             );
@@ -225,10 +255,10 @@ impl Action for AlterColumn {
             // the existing constraint for correctness which makes the lock short-lived.
             // Source: https://dba.stackexchange.com/a/268128
             let query = format!(
-                "
-                ALTER TABLE {table}
-                ALTER COLUMN {column} SET NOT NULL
-                ",
+                r#"
+                ALTER TABLE "{table}"
+                ALTER COLUMN "{column}" SET NOT NULL
+                "#,
                 table = self.table,
                 column = self.temporary_column_name(ctx),
             );
@@ -236,10 +266,10 @@ impl Action for AlterColumn {
 
             // Drop the temporary constraint
             let query = format!(
-                "
-                ALTER TABLE {table}
-                DROP CONSTRAINT {constraint_name}
-                ",
+                r#"
+                ALTER TABLE "{table}"
+                DROP CONSTRAINT "{constraint_name}"
+                "#,
                 table = self.table,
                 constraint_name = self.not_null_constraint_name(ctx),
             );
@@ -247,21 +277,64 @@ impl Action for AlterColumn {
                 .context("failed to drop NOT NULL constraint")?;
         }
 
+        // Replace old indices with the new temporary ones created for the temporary column
+        let indices = common::get_indices_for_column(db, &self.table, &self.column)?;
+        for (current_index_name, index_oid) in indices {
+            // To keep the index handling idempotent, we need to do the following:
+            // 1. Add a prefix to the existing index
+            // 2. Rename temporary index to its final name
+            // 3. Drop existing index concurrently
+
+            // Add prefix (if not already added) to existing index
+            let prefix = "__reshape_old";
+            let target_index_name = current_index_name.trim_start_matches(prefix);
+            let old_index_name = format!("{}_{}", prefix, target_index_name);
+            db.query(&format!(
+                r#"
+                ALTER INDEX IF EXISTS "{current_name}" RENAME TO "{new_name}"
+                "#,
+                current_name = target_index_name,
+                new_name = old_index_name,
+            ))
+            .context("failed to rename old index")?;
+
+            // Rename temporary index to real name
+            let temp_index_name = self.temp_index_name(ctx, index_oid);
+            db.query(&format!(
+                r#"
+                ALTER INDEX IF EXISTS "{temp_index_name}" RENAME TO "{target_index_name}"
+                "#,
+                temp_index_name = temp_index_name,
+                target_index_name = target_index_name,
+            ))
+            .context("failed to rename temporary index")?;
+
+            // Drop old index concurrently
+            db.query(&format!(
+                r#"
+                DROP INDEX CONCURRENTLY IF EXISTS "{old_index_name}"
+                "#,
+                old_index_name = old_index_name,
+            ))
+            .context("failed to drop old index")?;
+        }
+
         // Remove old column
         let query = format!(
-            "
-            ALTER TABLE {} DROP COLUMN {} CASCADE
-			",
-            self.table, self.column
+            r#"
+            ALTER TABLE "{table}" DROP COLUMN IF EXISTS "{column}" CASCADE
+			"#,
+            table = self.table,
+            column = self.column,
         );
         db.run(&query).context("failed to drop old column")?;
 
         // Rename temporary column
         let column_name = self.changes.name.as_deref().unwrap_or(&self.column);
         let query = format!(
-            "
-            ALTER TABLE {table} RENAME COLUMN {temp_column} TO {name}
-			",
+            r#"
+            ALTER TABLE "{table}" RENAME COLUMN "{temp_column}" TO "{name}"
+			"#,
             table = self.table,
             temp_column = self.temporary_column_name(ctx),
             name = column_name,
@@ -271,13 +344,13 @@ impl Action for AlterColumn {
 
         // Remove triggers and procedures
         let query = format!(
-            "
-            DROP TRIGGER IF EXISTS {up_trigger} ON {table};
-            DROP FUNCTION IF EXISTS {up_trigger};
+            r#"
+            DROP TRIGGER IF EXISTS "{up_trigger}" ON "{table}";
+            DROP FUNCTION IF EXISTS "{up_trigger}";
 
-            DROP TRIGGER IF EXISTS {down_trigger} ON {table};
-            DROP FUNCTION IF EXISTS {down_trigger};
-            ",
+            DROP TRIGGER IF EXISTS "{down_trigger}" ON "{table}";
+            DROP FUNCTION IF EXISTS "{down_trigger}";
+            "#,
             table = self.table,
             up_trigger = self.up_trigger_name(ctx),
             down_trigger = self.down_trigger_name(ctx),
@@ -311,12 +384,25 @@ impl Action for AlterColumn {
     }
 
     fn abort(&self, ctx: &MigrationContext, db: &mut dyn Conn) -> anyhow::Result<()> {
+        // Safely remove any indices created for the temporary column
+        let temp_column_name = self.temporary_column_name(ctx);
+        let indices = common::get_indices_for_column(db, &self.table, &temp_column_name)?;
+        for (_, index_oid) in indices {
+            let temp_index_name = self.temp_index_name(ctx, index_oid);
+            db.query(&format!(
+                r#"
+                DROP INDEX CONCURRENTLY IF EXISTS "{index_name}"
+                "#,
+                index_name = temp_index_name,
+            ))?;
+        }
+
         // Drop temporary column
         let query = format!(
-            "
-			ALTER TABLE {table}
-            DROP COLUMN IF EXISTS {temp_column};
-			",
+            r#"
+			ALTER TABLE "{table}"
+            DROP COLUMN IF EXISTS "{temp_column}";
+			"#,
             table = self.table,
             temp_column = self.temporary_column_name(ctx),
         );
@@ -324,13 +410,13 @@ impl Action for AlterColumn {
 
         // Remove triggers and procedures
         let query = format!(
-            "
-            DROP TRIGGER IF EXISTS {up_trigger} ON {table};
-            DROP FUNCTION IF EXISTS {up_trigger};
+            r#"
+            DROP TRIGGER IF EXISTS "{up_trigger}" ON "{table}";
+            DROP FUNCTION IF EXISTS "{up_trigger}";
 
-            DROP TRIGGER IF EXISTS {down_trigger} ON {table};
-            DROP FUNCTION IF EXISTS {down_trigger};
-            ",
+            DROP TRIGGER IF EXISTS "{down_trigger}" ON "{table}";
+            DROP FUNCTION IF EXISTS "{down_trigger}";
+            "#,
             table = self.table,
             up_trigger = self.up_trigger_name(ctx),
             down_trigger = self.down_trigger_name(ctx),
@@ -359,6 +445,10 @@ impl AlterColumn {
         format!("{}_alter_column_temporary", ctx.prefix())
     }
 
+    fn temp_index_name(&self, ctx: &MigrationContext, index_oid: u32) -> String {
+        format!("{}_alter_column_temp_index_{}", ctx.prefix(), index_oid)
+    }
+
     fn can_short_circuit(&self) -> bool {
         self.changes.name.is_some()
             && self.changes.data_type.is_none()
