@@ -1,3 +1,4 @@
+use anyhow::anyhow;
 use derive_builder::Builder;
 use postgres::types::{FromSql, ToSql};
 use serde::{Deserialize, Serialize};
@@ -87,7 +88,9 @@ pub fn batch_touch_rows(db: &mut dyn Conn, table: &str, column: &str) -> anyhow:
             .iter()
             .map(|column| {
                 format!(
-                    "{table}.{column} = rows.{column}",
+                    r#"
+                    "{table}"."{column}" = rows."{column}"
+                    "#,
                     table = table,
                     column = column,
                 )
@@ -97,7 +100,7 @@ pub fn batch_touch_rows(db: &mut dyn Conn, table: &str, column: &str) -> anyhow:
 
         let returning_columns = primary_key
             .iter()
-            .map(|column| format!("rows.{}", column))
+            .map(|column| format!("rows.\"{}\"", column))
             .collect::<Vec<String>>()
             .join(", ");
 
@@ -113,24 +116,24 @@ pub fn batch_touch_rows(db: &mut dyn Conn, table: &str, column: &str) -> anyhow:
         };
 
         let query = format!(
-            "
-                WITH rows AS (
-                    SELECT {primary_key_columns}
-                    FROM public.{table}
-                    {cursor_where}
-                    ORDER BY {primary_key_columns}
-                    LIMIT {batch_size}
-                ), update AS (
-                    UPDATE public.{table}
-                    SET {column} = {column}
-                    FROM rows
-                    WHERE {primary_key_where}
-                    RETURNING {returning_columns}
-                )
-                SELECT LAST_VALUE(({primary_key_columns})) OVER () AS last_value
-                FROM update
-                LIMIT 1
-			    ",
+            r#"
+            WITH rows AS (
+                SELECT {primary_key_columns}
+                FROM public."{table}"
+                {cursor_where}
+                ORDER BY {primary_key_columns}
+                LIMIT {batch_size}
+            ), update AS (
+                UPDATE public."{table}"
+                SET "{column}" = "{column}"
+                FROM rows
+                WHERE {primary_key_where}
+                RETURNING {returning_columns}
+            )
+            SELECT LAST_VALUE(({primary_key_columns})) OVER () AS last_value
+            FROM update
+            LIMIT 1
+            "#,
             table = table,
             primary_key_columns = primary_key_columns,
             cursor_where = cursor_where,
@@ -178,3 +181,79 @@ fn get_primary_key_columns_for_table(
 
     Ok(primary_key_columns)
 }
+
+pub fn get_indices_for_column(
+    db: &mut dyn Conn,
+    table: &str,
+    column: &str,
+) -> anyhow::Result<Vec<(String, u32)>> {
+    let indices = db
+        .query(&format!(
+            "
+            SELECT i.relname AS index_name, i.oid AS index_oid
+            FROM pg_index ix
+            JOIN pg_class t ON t.oid = ix.indrelid
+            JOIN pg_class i ON i.oid = ix.indexrelid
+            JOIN pg_attribute a ON
+                a.attrelid = t.oid AND
+                a.attnum = ANY(ix.indkey)
+            WHERE
+                t.relname = '{table}' AND
+                a.attname = '{column}'
+            ",
+            table = table,
+            column = column,
+        ))?
+        .iter()
+        .map(|row| (row.get("index_name"), row.get("index_oid")))
+        .collect();
+
+    Ok(indices)
+}
+
+pub fn get_index_columns(db: &mut dyn Conn, index_name: &str) -> anyhow::Result<Vec<String>> {
+    // Get all columns which are part of the index in order
+    let (table_oid, column_nums) = db
+        .query(&format!(
+            "
+            SELECT t.oid AS table_oid, ix.indkey::INTEGER[] AS columns
+            FROM pg_index ix
+            JOIN pg_class t ON t.oid = ix.indrelid
+            JOIN pg_class i ON i.oid = ix.indexrelid
+            WHERE
+	            i.relname = '{index_name}'
+            ",
+            index_name = index_name,
+        ))?
+        .first()
+        .map(|row| {
+            (
+                row.get::<'_, _, u32>("table_oid"),
+                row.get::<'_, _, Vec<i32>>("columns"),
+            )
+        })
+        .ok_or_else(|| anyhow!("failed to get columns for index"))?;
+
+    // Get the name of each of the columns, still in order
+    column_nums
+        .iter()
+        .map(|column_num| -> anyhow::Result<String> {
+            let name: String = db
+                .query(&format!(
+                    "
+                    SELECT attname AS name
+                    FROM pg_attribute
+                    WHERE attrelid = {table_oid}
+                        AND attnum = {column_num};
+                    ",
+                    table_oid = table_oid,
+                    column_num = column_num,
+                ))?
+                .first()
+                .map(|row| row.get("name"))
+                .ok_or_else(|| anyhow!("expected to find column"))?;
+
+            Ok(name)
+        })
+        .collect::<anyhow::Result<Vec<String>>>()
+}
