@@ -1,5 +1,4 @@
 use reshape::migrations::{AddColumn, Column, ColumnBuilder, CreateTableBuilder, Migration};
-use reshape::Status;
 
 mod common;
 
@@ -59,11 +58,6 @@ fn add_column() {
 
     // Run first migration, should automatically finish
     reshape.migrate(first_migrations.clone()).unwrap();
-    assert!(matches!(reshape.state.status, Status::Idle));
-    assert_eq!(
-        Some(&create_users_table.name),
-        reshape.state.current_migration.as_ref()
-    );
 
     // Update search paths
     old_db
@@ -118,7 +112,7 @@ fn add_column() {
         (first_name.as_ref(), last_name.as_ref())
     );
 
-    reshape.complete_migration().unwrap();
+    reshape.complete().unwrap();
     common::assert_cleaned_up(&mut new_db);
 }
 
@@ -155,11 +149,6 @@ fn add_column_nullable() {
 
     // Run first migration, should automatically finish
     reshape.migrate(first_migrations.clone()).unwrap();
-    assert!(matches!(reshape.state.status, Status::Idle));
-    assert_eq!(
-        Some(&create_users_table.name),
-        reshape.state.current_migration.as_ref()
-    );
 
     // Update search paths
     old_db
@@ -216,7 +205,7 @@ fn add_column_nullable() {
         .simple_query("INSERT INTO users (id, name) VALUES (4, 'Test Testsson'), (5, NULL)")
         .unwrap();
 
-    reshape.complete_migration().unwrap();
+    reshape.complete().unwrap();
     common::assert_cleaned_up(&mut new_db);
 }
 
@@ -254,11 +243,6 @@ fn add_column_with_default() {
 
     // Run first migration, should automatically finish
     reshape.migrate(first_migrations.clone()).unwrap();
-    assert!(matches!(reshape.state.status, Status::Idle));
-    assert_eq!(
-        Some(&create_users_table.name),
-        reshape.state.current_migration.as_ref()
-    );
 
     // Update search paths
     old_db
@@ -304,6 +288,6 @@ fn add_column_with_default() {
         .unwrap();
     assert_eq!("DEFAULT", name);
 
-    reshape.complete_migration().unwrap();
+    reshape.complete().unwrap();
     common::assert_cleaned_up(&mut new_db);
 }
