@@ -1,7 +1,4 @@
-use reshape::{
-    migrations::{ColumnBuilder, CreateTableBuilder, ForeignKey, Migration},
-    Status,
-};
+use reshape::migrations::{ColumnBuilder, CreateTableBuilder, ForeignKey, Migration};
 
 mod common;
 
@@ -40,11 +37,6 @@ fn create_table() {
     reshape
         .migrate(vec![create_table_migration.clone()])
         .unwrap();
-    assert!(matches!(reshape.state.status, Status::Idle));
-    assert_eq!(
-        Some(&create_table_migration.name),
-        reshape.state.current_migration.as_ref()
-    );
 
     // Ensure table was created
     let result = db
@@ -176,20 +168,20 @@ fn create_table_with_foreign_keys() {
     let foreign_key_columns: Vec<(String, String, String)> = db
         .query(
             "
-        SELECT
-            kcu.column_name, 
-            ccu.table_name AS foreign_table_name,
-            ccu.column_name AS foreign_column_name 
-        FROM 
-            information_schema.table_constraints AS tc 
-            JOIN information_schema.key_column_usage AS kcu
-              ON tc.constraint_name = kcu.constraint_name
-              AND tc.table_schema = kcu.table_schema
-            JOIN information_schema.constraint_column_usage AS ccu
-              ON ccu.constraint_name = tc.constraint_name
-              AND ccu.table_schema = tc.table_schema
-        WHERE tc.constraint_type = 'FOREIGN KEY' AND tc.table_name='items';
-        ",
+            SELECT
+                kcu.column_name, 
+                ccu.table_name AS foreign_table_name,
+                ccu.column_name AS foreign_column_name 
+            FROM 
+                information_schema.table_constraints AS tc 
+                JOIN information_schema.key_column_usage AS kcu
+                ON tc.constraint_name = kcu.constraint_name
+                AND tc.table_schema = kcu.table_schema
+                JOIN information_schema.constraint_column_usage AS ccu
+                ON ccu.constraint_name = tc.constraint_name
+                AND ccu.table_schema = tc.table_schema
+            WHERE tc.constraint_type = 'FOREIGN KEY' AND tc.table_name='items';
+            ",
             &[],
         )
         .unwrap()
